<?php
/****************************************************
  Coppermine 1.5.x Plugin - Facebook OAuth (fb_oauth)
  ***************************************************
  Copyright (c) 2010 Jon Troncoso, 2011 Roland Fulde
  **************************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

// We may or may not have this data based on a $_GET or $_COOKIE based session.
//
// If we get a session here, it means we found a correctly signed session using
// the Application Secret only Facebook and the Application know. We dont know
// if it is still valid until we make an API call using the session. A session
// can become invalid if it has already expired (should not be getting the
// session back in this case) or if the user logged out of Facebook.

$thisplugin->add_action('plugin_install','fb_oauth_install');
$thisplugin->add_action('plugin_uninstall','fb_oauth_uninstall');
$thisplugin->add_action('page_start','fboa_init');
$thisplugin->add_filter('sys_menu','add_syb_button');


// Install the plugin
function fb_oauth_install() {
	global $CONFIG, $thisplugin;

	// about the table
	// -- be generous about the Facebook uid, you never know how logn Facebook might make it
	// -- the FOREIGN KEY REFERENCES CASCADE clauses seem to be ignored completely; in mysql docs it sounds as if that only works with InnoDB?
	$query = <<< EOT
		CREATE TABLE IF NOT EXISTS {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap (
			cpg_user_id int(11) NOT NULL,
		  	fb_uid varchar(255) NOT NULL UNIQUE,
		  	CONSTRAINT FOREIGN KEY (cpg_user_id)
		  	     REFERENCES {$CONFIG['TABLE_USERS']} (user_id)
		  	     ON DELETE CASCADE ON UPDATE CASCADE
  	  	) COMMENT='maps Facebook uid to Coppermine user_id';
EOT;
	$result = cpg_db_query($query);	
	return true;
}

// Uninstall the plugin
function fb_oauth_uninstall() {
	global $CONFIG, $thisplugin;

	$query = <<< EOT
		DROP TABLE `{$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap`
EOT;
	$result = cpg_db_query($query);	
	return true;
}

function add_syb_button($menu) {
	require("./plugins/fb_oauth/fb_init.php");
	global $CONFIG, $template_sys_menu_spacer, $USER_DATA, $USER;
	$superCage = Inspekt::makeSuperCage();
	
    // Include the language file
//    require_once ("./plugins/fb_oauth/lang/english.php");
//    if (file_exists("./plugins/fb_oauth/lang/{$USER['lang']}.php")) {
//        require_once ("./plugins/fb_oauth/lang/{$USER['lang']}.php");
//    } 
    	
	$me = null;
	// Session based API call.
	if (!empty($session)) {
	  try {
	    $uid = $facebook->getUser();
	    $me = $facebook->api('/me');
	  } catch (FacebookApiException $e) {
	    error_log($e);
	  }
	}

	// add buttons only when the user is not logged on or logged on via facebook
	if (!$USER_DATA['user_id'] || $me) {
	    // login or logout url will be needed depending on current user state.
	    if ($me) { // logout button, do not add, just change the existing one
                $logoutButton = find_button($menu, '{LOGOUT_LNK}');
                $buttonLnk = $facebook->getLogoutURL();
		$buttonTtl = '<img src="https://graph.facebook.com/'.$uid.'/picture" border="0" height="10" style="height:1em;" alt="" class="icon" />';
                              // style= with "em" sizing is relative to font size; the old height attribute actually not needed anymore.
		$buttonHelp = $lang_plugin_fb_oauth['logout_title'];
                $menu[$logoutButton][0] = $buttonTtl.$menu[$logoutButton][0];
		$menu[$logoutButton][1] = $buttonHelp;
		$menu[$logoutButton][2] = $buttonLnk;
            } else { // login button, add one and modify the existing one to avoid confusion
                $loginButton = find_button($menu, '{LOGIN_LNK}');
	        $buttonLnk = $facebook->getLoginUrl($login_params);
		$buttonTtl = '<img src="./plugins/fb_oauth/16x16.png" border="0" height="12" style="height:1em;" alt="" class="icon" />';
		$buttonTtl .= $lang_plugin_fb_oauth['login_lnk'];
		$buttonHelp = $lang_plugin_fb_oauth['login_title'];
		$menu[$loginButton][0] = $lang_plugin_fb_oauth['normal_login_lnk'];

		$new_button = array();
		$new_button[0][0] = $buttonTtl;
		$new_button[0][1] = $buttonHelp;
		$new_button[0][2] = $buttonLnk;
		$new_button[0][3] = 'fb_oauth';
		$new_button[0][4] = $template_sys_menu_spacer;
		$new_button[0][5] = 'rel="nofollow"';

		array_splice($menu, $loginButton, 0, $new_button);		
            }
  	}
	return $menu;
}

function find_button($menu, $whichOne, $byWhat=0) {
    foreach($menu as $bkey => $button) {
        if ($button[$byWhat] == $whichOne) {
            return $bkey;
        }
    }
}

function fboa_init() {
	require("./plugins/fb_oauth/fb_init.php");
	global $CONFIG, $JS, $USER_DATA, $cpg_udb;

	$JS['includes'][] = 'plugins/fb_oauth/include/fbconnect.js';
	
	if($USER_DATA['user_id']!=0 && empty($session))return false;
	$me = null;
	// Session based API call.
	if (!empty($session)) {
	  try {
	    $uid = $facebook->getUser();
	    $uid = mysql_real_escape_string ($uid);	// We will use it in SQL String here (and only in those!)
	    $me = $facebook->api('/me');
	  } catch (FacebookApiException $e) {
	    error_log($e);
	  }
		//Log out if you detect a facebook logout
		if($USER_DATA['user_id']!=0){

			if(!$me){
				$cpg_udb->logout();
				cpgRedirectPage('index.php', $lang_plugin_fb_oauth['after_fb_logout_msg'], $lang_plugin_fb_oauth['after_fb_logout_msg'], 3);
			}
			return false;
		}

		if($me){
			$needtocreateuser = true;
			$sql = "SELECT cpg_user_id FROM {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap WHERE fb_uid='$uid'";
			$result = cpg_db_query($sql);
			if(mysql_num_rows($result) > 0) {
				$row = mysql_fetch_array($result);
			        $cpguserid = $row['cpg_user_id'];
				mysql_free_result($result);
				
				$sql = "SELECT user_id FROM {$CONFIG['TABLE_USERS']} WHERE user_id = $cpguserid";
				$result = cpg_db_query($sql);
				if(mysql_num_rows($result) > 0) {
				      // we have the mapping and we have the user.
				      $needtocreateuser = false;
				} else {
				      // dammit, we have the user in the mapping table but not in Coppermine.
				      // Maybe he has been deleted earlier but the mapping did not go away
				      // (see comment above about MySQL's FOREIGN KEY ... CASCADE clause).
				      // I suppose it is best/cleanest to then also delete the old mapping entry
				      // and start over like with anyone else
				      $sql = "DELETE FROM {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap WHERE fb_uid = '$uid'";
				      $result = cpg_db_query($sql);
				      $needtocreateuser = true;
				}
			}
			if ($needtocreateuser) {
			        $user_language = select_language($me['locale']);
				$attempt=-1;
				while(1) {
				        // Let's generate a Coppermine username from the Facebook username in a Really Good(tm) way.
				        // Want to cope also with possible multiple different people all named John Doe, who all do not share their Birthdate from Facebook,
				        // So first trying without birthdate, then with, and if that still does not work, find a number to append.
				        // Note well that due to the mapping table, a Coppermine administrator can change the Coppermine username for this user without any problem.
					switch($attempt) {
						case -1:
							$usern = $me['first_name'].$me['last_name'];
							break;
						case 0:
							$usern = $me['first_name'].$me['last_name'].preg_replace('/\//', '', $me['birthday']);
							break;
						default:
							$usern = $me['first_name'].$me['last_name'].$attempt;
							break;
					}
					$sql = "SELECT user_id FROM {$CONFIG['TABLE_USERS']} WHERE user_name = '{$usern}'";
					$result = cpg_db_query($sql);
					if(mysql_num_rows($result) == 0) {
						break;
					}
					$attempt++;
				}

				// Random password -- yeah, we want that for security
				// If the user who originally came from Facebook wants to change it
				// in his Coppermine profile, he is free to do so.
				$password = genRandomString(40);
			        $encpassword = md5($password);

				$sql = "SELECT group_id FROM {$CONFIG['TABLE_USERGROUPS']} WHERE group_name = '{$fb_defaultgroup}'";
				$result = cpg_db_query($sql);
				if(mysql_num_rows($result) > 0) {
					$row = mysql_fetch_array($result);
					$groupid = $row['group_id'];
					mysql_free_result($result);
					$sql = "INSERT INTO {$CONFIG['TABLE_USERS']} (user_regdate, user_active, user_actkey, user_name, user_password, user_group, user_email, user_profile1, user_profile2, user_profile3, user_profile4, user_profile5, user_profile6, user_language) VALUES (NOW(), 'YES', '', '$usern', '$encpassword', '$groupid', '{$me['email']}', '', '', '', '', '', '', '$user_language')";
				} else {
					$sql = "INSERT INTO {$CONFIG['TABLE_USERS']} (user_regdate, user_active, user_actkey, user_name, user_password, user_email, user_profile1, user_profile2, user_profile3, user_profile4, user_profile5, user_profile6, user_language) VALUES (NOW(), 'YES', '', '$usern', '$encpassword', '{$me['email']}', '', '', '', '', '', '', '$user_language')";
				}
				$result = cpg_db_query($sql);

				$cpguserid = mysql_insert_id();
				$sql="INSERT INTO {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap (cpg_user_id, fb_uid) values ($cpguserid, '$uid')";
				$result = cpg_db_query($sql);
			}
			$USER_DATA = fb_oauth_direct_login ($cpguserid);
			if($USER_DATA['user_id']) {
				cpgRedirectPage('index.php', $lang_plugin_fb_oauth['after_fb_login_msg'], $lang_plugin_fb_oauth['after_fb_login_msg'], 3);
			} else {
				$cpg_udb->logout();
                                msg_box('', $lang_plugin_fb_oauth['fb_login_fail'], '', '', 'error');
			}
		}
	}
	
}

function genRandomString($length) {
	$characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$string = '';
	for ($i = 0; $i < $length; $i++) {
		$string .= $characters[mt_rand(0, strlen($characters))];
	}
	return $string;
}


function select_language($code) {
    global $USER;
    $code = strtolower($code);
    switch (substr($code, 0, 4)) {
        case 'en-gb':
        case 'en_gb': return 'english_gb';
        case 'pt-br':
        case 'pt_br': return 'brazilian_portuguese';
        case 'zh-tw':
        case 'zh_tw': return 'chinese_big5';
        default:
            switch (substr($code, 0, 2)) {
                case 'af': return 'afrikaans';
                case 'ar': return 'arabic';
                case 'be': return 'belarusian';
                case 'bg': return 'bulgarian';
                case 'ca': return 'catalan';
                case 'cs': return 'czech';
                case 'da': return 'danish';
                case 'de': return 'german';
                case 'el': return 'greek';
                case 'en': return 'english';
                case 'es': return 'spanish';
                case 'et': return 'estonian';
                case 'fa': return 'farsi';
                case 'fi': return 'finnish';
                case 'fr': return 'french';
                case 'gl': return 'galician';
                case 'he': return 'hebrew';
                case 'hi': return 'hindi';
                case 'hr': return 'croatian';
                case 'hu': return 'hungarian';
                case 'id': return 'indonesian';
                case 'is': return 'icelandic';
                case 'it': return 'italian';
                case 'ja': return 'japanese';
                case 'ko': return 'korean';
                case 'ka': return 'georgian';
                case 'kk': return 'kazakh';
                case 'ls': return 'slovenian';
                case 'lt': return 'lithuanian';
                case 'lu': return 'luxembourgish';
                case 'lv': return 'latvian';
                case 'ms': return 'malay';
                case 'nl': return 'dutch';
                case 'no': return 'norwegian';
                case 'pl': return 'polish';
                case 'pt': return 'portuguese';
                case 'ro': return 'romanian';
                case 'ru': return 'russian';
                case 'sk': return 'slovak';
                case 'sl': return 'slovenian';
                case 'sq': return 'albanian';
                case 'sr': return 'serbian';
                case 'sv': return 'swedish';
                case 'th': return 'thai';
                case 'tr': return 'turkish';
                case 'uk': return 'ukrainian';
                case 'zh': return 'chinese_gb';
                default: return $USER['lang'];
            }
    }
}

// imitate what bridge/coppermine.inc.php does for login()
function fb_oauth_direct_login($cpguserid) {
                    global $CONFIG, $cpg_udb;

                    // Create the session_id from concat(cookievalue,client_id)
                    $session_id = $cpg_udb->session_id.$cpg_udb->client_id;

                    // Check for user in users table
                    $sql =  "SELECT user_id, user_name, user_password FROM {$CONFIG['TABLE_USERS']} WHERE user_id = $cpguserid AND user_active = 'YES'";
                    $results = cpg_db_query($sql);

                    // If exists update lastvisit value, session, and login
                    if (mysql_num_rows($results)) {

                            // Update lastvisit value
                            $sql =  "UPDATE {$CONFIG['TABLE_USERS']} SET user_lastvisit = NOW() WHERE user_id = $cpguserid AND user_active = 'YES'";
                            cpg_db_query($sql, $cpg_udb->link_id);

                            $USER_DATA = mysql_fetch_assoc($results);
                            mysql_free_result($results);

                            setcookie( $cpg_udb->client_id, $cpg_udb->session_id, 0, $CONFIG['cookie_path'] );

                            // Update guest session with user's information
                            $sql  = "UPDATE {$CONFIG['TABLE_PREFIX']}sessions SET ";
                            $sql .= "user_id = {$USER_DATA['user_id']} ";
                            $sql .= "WHERE session_id = '" . md5($session_id) . "'";
                            cpg_db_query($sql, $cpg_udb->link_id);

                            return $USER_DATA;
                    } else {

                            return false;
                    }
            }
?>
